<?php

namespace Growmatik\Form_Integration\Elementor;

use ElementorPro\Modules\Forms\Classes\Action_Base;
use Elementor\Repeater;
use Elementor\Controls_Manager;
use Growmatik\Form_Integration\Integration;

class Elementor_Form_Growmatik_Action extends Action_Base {

	private $growmatik_tags, $attributes_config, $form_fields_config; //phpcs:ignore
	private $all_attributes = array();

	public function get_name() {
		return 'growmatik';
	}

	public function get_label() {
		return __( 'Growmatik', 'growmatik' );
	}

	public function update_growmatik_attributes() {
		$this->all_attributes = Integration::instance()->get_all_attributes();
		$this->growmatik_tags = Integration::instance()->get_tags();
	}

	private function set_controls_config() {

		$this->attributes_config = array(
			'name'    => 'growmatik_attribute',
			'label'   => __( 'Attribute', 'growmatik' ),
			'type'    => Controls_Manager::SELECT,
			'options' => array( 'none' => __( '- None -', 'growmatik' ) ) + $this->all_attributes,
			'default' => 'none',
		);

		$this->form_fields_config = array(
			'name'    => 'growmatik_form_field',
			'label'   => __( 'Form Field', 'growmatik' ),
			'type'    => Controls_Manager::SELECT,
			'options' => array(// Will be updated by JS while changing form fields.
				'none' => __( '- None -', 'growmatik' ),
			),
			'default' => 'none',
		);
	}

	private function register_controls_by_add_control() {
		$repeater = new Repeater();

		$repeater->add_control(
			'growmatik_attribute',
			$this->attributes_config
		);

		$repeater->add_control(
			'growmatik_form_field',
			$this->form_fields_config
		);

		return $repeater->get_controls();
	}

	/**
	 * Add Growmatik settings section in Elementor form widget.
	 */
	public function register_settings_section( $element ) {
		$this->update_growmatik_attributes();
		$this->set_controls_config();

		$element->start_controls_section(
			'section_growmatik',
			array(
				'tab'       => 'content',
				'label'     => __( 'Growmatik', 'growmatik' ),
				'condition' => array(
					'submit_actions' => $this->get_name(),
				),
			)
		);

		$element->add_control(
			'growmatik_form_name',
			array(
				'type'    => Controls_Manager::TEXT,
				'label'   => __( 'Form Name', 'gromatik' ),
				'default' => __( 'New Form', 'growmatik' ),
			)
		);

		$element->add_control(
			'growmatik_form_tags',
			array(
				'type'        => Controls_Manager::SELECT2,
				'label'       => __( 'Tags', 'growmatik' ),
				'description' => __( 'Add comma separated tags', 'growmatik' ),
				'multiple'    => true,
				'options'     => $this->growmatik_tags,
			)
		);

		$element->add_control(
			'gromatik_heading_field_mapping',
			array(
				'type'  => Controls_Manager::HEADING,
				'label' => __( 'Field Mapping', 'growmatik' ),
			)
		);

		$repeater_control_args = array(
			'name'          => 'growmatik_mapping_fields',
			'type'          => Controls_Manager::REPEATER,
			'title_field'   => '{{ growmatik_form_field }}',
			'prevent_empty' => true,
			'default'       => array(
				array(
					'growmatik_attribute'  => 'none',
					'growmatik_form_field' => 'none',
				),
			),
		);

		$repeater_control_args['fields'] = $this->register_controls_by_add_control();

		$element->add_control(
			'growmatik_mapping_fields',
			$repeater_control_args
		);

		$element->end_controls_section();
	}

	/**
	 * Run
	 *
	 * Runs the action after submit
	 *
	 * @access public
	 * @param \ElementorPro\Modules\Forms\Classes\Form_Record  $record
	 * @param \ElementorPro\Modules\Forms\Classes\Ajax_Handler $ajax_handler
	 */
	public function run( $record, $ajax_handler ) {
		$this->update_growmatik_attributes();

		$settings   = $record->get( 'form_settings' );
		$raw_fields = $record->get( 'fields' );
		$all_fields = array();
		$site_id    = get_option( 'growmatik_site_id' );
		$tags       = $settings['growmatik_form_tags'];
		$form_name  = $settings['growmatik_form_name'];
		$vals       = array( 'on', true, 'true', 'checked', '1', 1, 'granted' );

		// Get value of default and custom mapped fields.
		foreach ( $settings['growmatik_mapping_fields'] as $field_key => $value ) {
			if ( ! isset( $value['growmatik_form_field'] ) ) {
				continue;
			}

			$field_id = $value['growmatik_form_field'];

			if ( $field_id ) {
				$growmatik_field = str_replace( 'growmatik-', '', $value['growmatik_attribute'] );

				if ( ! empty( $growmatik_field ) && 'none' !== $growmatik_field ) {
					if ( in_array( $growmatik_field, array( '012', '013' ), true ) ) {
						$raw_fields[ $field_id ]['value'] = in_array( $raw_fields[ $field_id ]['value'], $vals, true )
						? 'granted' : 'denied';
					}

					$field = array(
						'key'   => $growmatik_field,
						'value' => $raw_fields[ $field_id ]['value'],
						'type'  => Integration::instance()->get_attribute_type( $growmatik_field ),
					);

					array_push( $all_fields, $field );
				}
			}
		}

		$form_data = array(
			'siteId'   => $site_id,
			'formName' => $form_name,
			'tags'     => $tags,
			'formData' => $all_fields,
		);

		$response = Integration::instance()->form_submission( $form_data );

		if ( is_wp_error( $response ) ) {
			$ajax_handler->add_admin_error_message( 'Growmatik: Internal server error.' );
		} elseif ( ! $response['success'] ) {
			$ajax_handler->add_admin_error_message( 'Growmatik: Could not send the form successfuly.' );
		}
	}

	/**
	 * On Export
	 *
	 * Clears form settings on export
	 *
	 * @access public
	 * @param array $element
	 */
	public function on_export( $element ) {
		foreach ( $this->growmatik_fields as $fields_key => $label ) {
			if ( isset( $element[ 'growmatik_mapping_fields_' . $fields_key ] ) ) {
				unset( $element[ 'growmatik_mapping_fields_' . $fields_key ] );
			}
		}
	}
}
